@@ -21,6 +21,7 @@ from cliff import lister
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from osc_lib.utils import tags as _tag
 from oslo_utils import uuidutils
 
 from octaviaclient.osc.v2 import constants as const
@@ -30,14 +31,14 @@ HTTP_METHODS = ['GET', 'POST', 'DELETE', 'PUT', 'HEAD', 'OPTIONS', 'PATCH',
                 'CONNECT', 'TRACE']
 HTTP_VERSIONS = [1.0, 1.1]
 TYPE_CHOICES = ['PING', 'HTTP', 'TCP', 'HTTPS', 'TLS-HELLO',
-                'UDP-CONNECT']
+                'UDP-CONNECT', 'SCTP']
 
 
 class CreateHealthMonitor(command.ShowOne):
     """Create a health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(CreateHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'pool',
@@ -137,9 +138,12 @@ class CreateHealthMonitor(command.ShowOne):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
+        _tag.add_tag_option_to_parser_for_create(
+            parser, 'health monitor')
+
         return parser
 
     def take_action(self, parsed_args):
@@ -164,7 +168,8 @@ class CreateHealthMonitor(command.ShowOne):
                         data['healthmonitor']['id']))
             }
 
-        formatters = {'pools': v2_utils.format_list}
+        formatters = {'pools': v2_utils.format_list,
+                      'tags': v2_utils.format_list_flat}
 
         return (rows,
                 (utils.get_dict_properties(data['healthmonitor'],
@@ -176,7 +181,7 @@ class DeleteHealthMonitor(command.Command):
     """Delete a health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(DeleteHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'health_monitor',
@@ -186,7 +191,7 @@ class DeleteHealthMonitor(command.Command):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
         return parser
@@ -212,7 +217,9 @@ class ListHealthMonitor(lister.Lister):
     """List health monitors"""
 
     def get_parser(self, prog_name):
-        parser = super(ListHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
+
+        _tag.add_tag_filtering_option_to_parser(parser, 'health monitor')
 
         return parser
 
@@ -233,7 +240,7 @@ class ShowHealthMonitor(command.ShowOne):
     """Show the details of a single health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(ShowHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'health_monitor',
@@ -262,7 +269,8 @@ class ShowHealthMonitor(command.ShowOne):
             data = self.app.client_manager.load_balancer.health_monitor_show(
                 health_monitor_id=health_monitor_id,
             )
-        formatters = {'pools': v2_utils.format_list}
+        formatters = {'pools': v2_utils.format_list,
+                      'tags': v2_utils.format_list_flat}
 
         return (rows,
                 (utils.get_dict_properties(data, rows, formatters=formatters)))
@@ -272,7 +280,7 @@ class SetHealthMonitor(command.Command):
     """Update a health monitor"""
 
     def get_parser(self, prog_name):
-        parser = super(SetHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'health_monitor',
@@ -361,9 +369,11 @@ class SetHealthMonitor(command.Command):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
+        _tag.add_tag_option_to_parser_for_set(parser, 'health monitor')
+
         return parser
 
     def take_action(self, parsed_args):
@@ -372,6 +382,10 @@ class SetHealthMonitor(command.Command):
 
         hm_id = attrs.pop('health_monitor_id')
 
+        v2_utils.set_tags_for_set(
+            self.app.client_manager.load_balancer.health_monitor_show,
+            hm_id, attrs, clear_tags=parsed_args.no_tag)
+
         body = {'healthmonitor': attrs}
 
         self.app.client_manager.load_balancer.health_monitor_set(
@@ -389,7 +403,7 @@ class UnsetHealthMonitor(command.Command):
     """Clear health monitor settings"""
 
     def get_parser(self, prog_name):
-        parser = super(UnsetHealthMonitor, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
 
         parser.add_argument(
             'health_monitor',
@@ -435,19 +449,26 @@ class UnsetHealthMonitor(command.Command):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
+
+        _tag.add_tag_option_to_parser_for_unset(parser, 'health monitor')
+
         return parser
 
     def take_action(self, parsed_args):
         unset_args = v2_utils.get_unsets(parsed_args)
-        if not len(unset_args):
+        if not unset_args and not parsed_args.all_tag:
             return
 
         hm_id = v2_utils.get_resource_id(
             self.app.client_manager.load_balancer.health_monitor_list,
             'healthmonitors', parsed_args.health_monitor)
 
+        v2_utils.set_tags_for_unset(
+            self.app.client_manager.load_balancer.health_monitor_show,
+            hm_id, unset_args, clear_tags=parsed_args.all_tag)
+
         body = {'healthmonitor': unset_args}
 
         self.app.client_manager.load_balancer.health_monitor_set(
